{
  This project shows how keep zooming in TRichViewEdit even if the current
  screen DPI is changed.
  This project is for Delphi 10.3+.

  Projects for older version of Delphi can use Screen.PixelsPerInch instead of
  CurrentPPI, and they cannot react on the screen DPI change in
  OnAfterMonitorDpiChanged event.
}

unit MainFrm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, RVRulerBase, RVRuler, RVScroll,
  RichView, RVEdit, Vcl.StdCtrls, Vcl.ComCtrls,
  RVStyle, CRVData, RVRVData, RVDataRootProps;

type
  TfrmMain = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVRuler1: TRVRuler;
    RVStyle1: TRVStyle;
    TrackBar1: TTrackBar;
    Label1: TLabel;
    procedure TrackBar1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormAfterMonitorDpiChanged(Sender: TObject; OldDPI,
      NewDPI: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  RichViewEdit1.Clear;
  RichViewEdit1.AddNL('Hello world!', 0);
  RichViewEdit1.Format;
  TrackBar1Change(nil);
end;

{
  Note: using a trackbar for zooming large documents is not a good idea.
  The editor needs to be reformatted on each zoom change, it is not fast for
  large documents. And a trackbar generates events for intermedia zooming
  percents when a user switches from one zoom to another.
}

procedure TfrmMain.TrackBar1Change(Sender: TObject);
begin
  RichViewEdit1.DocumentPixelsPerInch := MulDiv(TrackBar1.Position, CurrentPPI, 100);
  RichViewEdit1.Reformat;
  RVRuler1.Zoom :=
    Round(100*RichViewEdit1.GetRealDocumentPixelsPerInch / RVRuler1.ScreenRes);
  RVRuler1.UpdateRulerMargins;
  Label1.Caption := IntToStr(TrackBar1.Position) + '%';
end;

procedure TfrmMain.FormAfterMonitorDpiChanged(Sender: TObject; OldDPI,
  NewDPI: Integer);
begin
  TrackBar1Change(nil);
end;

end.
